@@ -41,6 +41,7 @@ from sqlalchemy.exc import ProgrammingError
 from sqlalchemy.orm.exc import MultipleResultsFound
 from sqlalchemy.orm.exc import NoResultFound
 from sqlalchemy.orm.query import Query
+from sqlalchemy.sql.schema import Column
 from werkzeug.exceptions import BadRequest
 from werkzeug.exceptions import HTTPException
 from werkzeug.urls import url_quote_plus
@@ -102,13 +103,18 @@ def _is_msie8or9():
     """Returns ``True`` if and only if the user agent of the client making the
     request indicates that it is Microsoft Internet Explorer 8 or 9.
 
+    .. note::
+
+       We have no way of knowing if the user agent is lying, so we just make
+       our best guess based on the information provided.
+
     """
-    if request.user_agent is None or request.user_agent.version is None:
-        return False
-    ua = request.user_agent
     # request.user_agent.version comes as a string, so we have to parse it
-    ua_version = tuple(int(d) for d in ua.version.split('.'))
-    return ua.browser == 'msie' and (8, 0) <= ua_version < (10, 0)
+    version = lambda ua: tuple(int(d) for d in ua.version.split('.'))
+    return (request.user_agent is not None
+            and request.user_agent.version is not None
+            and request.user_agent.browser == 'msie'
+            and (8, 0) <= version(request.user_agent) < (10, 0))
 
 
 def create_link_string(page, last_page, per_page):
@@ -284,15 +290,12 @@ def _parse_includes(column_names):
     left and a dictionary mapping relation name to a list containing the names
     of fields on the related model which should be included.
 
-    `column_names` is either ``None`` or a list of strings. If it is ``None``,
-    the returned pair will be ``(None, None)``.
+    `column_names` must be a list of strings.
 
     If the name of a relation appears as a key in the dictionary, then it will
     not appear in the list.
 
     """
-    if column_names is None:
-        return None, None
     dotted_names, columns = partition(column_names, lambda name: '.' in name)
     # Create a dictionary mapping relation names to fields on the related
     # model.
@@ -315,15 +318,12 @@ def _parse_excludes(column_names):
     left and a dictionary mapping relation name to a list containing the names
     of fields on the related model which should be excluded.
 
-    `column_names` is either ``None`` or a list of strings. If it is ``None``,
-    the returned pair will be ``(None, None)``.
+    `column_names` must be a list of strings.
 
     If the name of a relation appears in the list then it will not appear in
     the dictionary.
 
     """
-    if column_names is None:
-        return None, None
     dotted_names, columns = partition(column_names, lambda name: '.' in name)
     # Create a dictionary mapping relation names to fields on the related
     # model.
@@ -573,10 +573,16 @@ class API(ModelView):
 
         """
         super(API, self).__init__(session, model, *args, **kw)
-        self.exclude_columns, self.exclude_relations = \
-            _parse_excludes(exclude_columns)
-        self.include_columns, self.include_relations = \
-            _parse_includes(include_columns)
+        if exclude_columns is None:
+            self.exclude_columns, self.exclude_relations = (None, None)
+        else:
+            self.exclude_columns, self.exclude_relations = _parse_excludes(
+                [self._get_column_name(column) for column in exclude_columns])
+        if include_columns is None:
+            self.include_columns, self.include_relations = (None, None)
+        else:
+            self.include_columns, self.include_relations = _parse_includes(
+                [self._get_column_name(column) for column in include_columns])
         self.include_methods = include_methods
         self.validation_exceptions = tuple(validation_exceptions or ())
         self.results_per_page = results_per_page
@@ -604,6 +610,33 @@ class API(ModelView):
         for preprocessor in self.preprocessors['PUT_MANY']:
             self.preprocessors['PATCH_MANY'].append(preprocessor)
 
+    def _get_column_name(self, column):
+        """Retrieve a column name from a column attribute of SQLAlchemy
+        model class, or a string.
+
+        Raises `TypeError` when argument does not fall into either of those
+        options.
+
+        Raises `ValueError` if argument is a column attribute that belongs
+        to an incorrect model class.
+
+        """
+        if hasattr(column, '__clause_element__'):
+            clause_element = column.__clause_element__()
+            if not isinstance(clause_element, Column):
+                msg = ('Column must be a string or a column attribute'
+                       ' of SQLAlchemy ORM class')
+                raise TypeError(msg)
+            model = column.class_
+            if model is not self.model:
+                msg = ('Cannot specify column of model %s'
+                       ' while creating API for model %s' % (
+                        model.__name__, self.model.__name__))
+                raise ValueError(msg)
+            return clause_element.key
+
+        return column
+
     def _add_to_relation(self, query, relationname, toadd=None):
         """Adds a new or existing related model to each model specified by
         `query`.
@@ -1366,6 +1399,7 @@ class API(ModelView):
             current_app.logger.exception(str(exception))
             return self._handle_validation_exception(exception)
         except (DataError, IntegrityError, ProgrammingError) as exception:
+            self.session.rollback()
             current_app.logger.exception(str(exception))
             return dict(message=type(exception).__name__), 400
 
